"use strict";
var WatchOptions = (function () {
    function WatchOptions() {
    }
    return WatchOptions;
}());
exports.WatchOptions = WatchOptions;
var VueTypescriptComponentData = (function () {
    function VueTypescriptComponentData() {
        this.props = {};
        this.watch = {};
    }
    return VueTypescriptComponentData;
}());
exports.VueTypescriptComponentData = VueTypescriptComponentData;
function getOrCreateComponent(target) {
    if (typeof target !== 'object') {
        throw 'VueTypescriptComponent decorator is only allowed for non-static members';
    }
    if (!target.hasOwnProperty('$$vueTypescriptComponentData')) {
        target.$$vueTypescriptComponentData = new VueTypescriptComponentData();
    }
    return target.$$vueTypescriptComponentData;
}
/** Use field as property
 * If field is initialized, sets default value and type. Otherwise mark as required
 */
function prop(options) {
    if (options === void 0) { options = {}; }
    return function (target, member) {
        getOrCreateComponent(target).props[member] = options;
    };
}
exports.prop = prop;
/** Use method(val, oldVal) to watch expression
 * The method is still part of the `method` object and it is possible to use multiple @watch
 * annotations to watch multiple expressions with one string
 */
function watch(expression, options) {
    if (options === void 0) { options = {}; }
    return function (target, member) {
        var wo = new WatchOptions();
        wo.expression = expression;
        wo.options = options;
        var watch = getOrCreateComponent(target).watch;
        if (!watch[member]) {
            watch[member] = [];
        }
        watch[member].push(wo);
    };
}
exports.watch = watch;
var lifecycleHooks = ['init', 'created', 'beforeCompile', 'compiled', 'ready',
    'attached', 'detached', 'beforeDestroy', 'destroyed'];
/** Create vueComponentOptions constructor property based on method/field annotations
 *  if provided, use options as the base value (.data is always overriden)
 */
function component(options) {
    if (options === void 0) { options = {}; }
    return function (cls) {
        var d = cls.prototype.$$vueTypescriptComponentData
            || new VueTypescriptComponentData();
        var obj = new cls();
        cls.vueComponentOptions = options;
        options.methods = options.methods || {};
        options.props = options.props || {};
        // to get rid of Index signature of object type implicitly has an 'any' type.
        var props = options.props;
        options.computed = options.computed || {};
        options.watch = options.watch || {};
        options.data = function () {
            var newData = new cls();
            var r = {};
            for (var _i = 0, _a = Object.getOwnPropertyNames(newData); _i < _a.length; _i++) {
                var n = _a[_i];
                if (!props[n] && n[0] !== '_' && n[0] !== '$') {
                    r[n] = newData[n];
                }
            }
            return r;
        };
        for (var _i = 0, _a = Object.getOwnPropertyNames(d.props); _i < _a.length; _i++) {
            var n = _a[_i];
            props[n] = d.props[n];
        }
        // create data; add default values and types to props with initialisers
        var _loop_1 = function(n) {
            if (props[n]) {
                if (props[n].default !== undefined) {
                    throw 'Property "' + n + '" has initialiser and PropOptions.default (use either or)';
                }
                if (typeof obj[n] === 'object') {
                    props[n].default = function () { return new cls()[n]; };
                }
                else {
                    props[n].default = obj[n];
                }
                if (props[n].type === undefined) {
                    props[n].type = Object.getPrototypeOf(obj[n]).constructor;
                }
            }
        };
        for (var _b = 0, _c = Object.getOwnPropertyNames(obj); _b < _c.length; _b++) {
            var n = _c[_b];
            _loop_1(n);
        }
        // set props without default value to required
        for (var n in props) {
            if (props[n].required === undefined && props[n].default !== undefined) {
                props[n].required = true;
            }
        }
        // methods
        for (var _d = 0, _e = Object.getOwnPropertyNames(cls.prototype); _d < _e.length; _d++) {
            var n = _e[_d];
            if (n === 'constructor' || n === '$$vueTypescriptComponentData') {
                continue;
            }
            var pd = Object.getOwnPropertyDescriptor(cls.prototype, n);
            if (pd.set) {
                options.computed[n] = { set: pd.set, get: pd.get };
            }
            else if (pd.get) {
                options.computed[n] = pd.get;
            }
            else if (lifecycleHooks.indexOf(n) !== -1) {
                options[n] = pd.value;
            }
            else {
                options.methods[n] = pd.value;
            }
        }
        // watch
        for (var _f = 0, _g = Object.getOwnPropertyNames(d.watch); _f < _g.length; _f++) {
            var n = _g[_f];
            if (!options.methods[n]) {
                throw '@watch must decorate a method: ' + n;
            }
            for (var _h = 0, _j = d.watch[n]; _h < _j.length; _h++) {
                var o = _j[_h];
                if (options.watch[o.expression]) {
                    throw 'duplicate watch expression: ' + o.expression;
                }
                o.options.handler = obj[n];
                options.watch[o.expression] = o.options;
            }
        }
    };
}
exports.component = component;
